# %%
from pathlib import Path

from aind_mri_targeting.headframe_rotations import headframe_centers_of_mass

# %%
# Input files
basepath = Path(r"Y:\ephys\persist\data\MRI\processed\771433")
mri_path = basepath / "771433_100.nii.gz"
seg_path = basepath / "771433_HeadframeHoles.seg.nrrd"

# Output directory
output_dir = basepath

# Optional mouse ID
mouse_id = 771433

# Whether to overwrite:
force = False

# Default key format is "{}_{}" for orientation and AP direction.
# Must match the segments in the segmentation file. If not, specify the key
# format.
segment_format = None

ignore_list = []  # list of segment names to ignore
# %%
headframe_centers_of_mass(
    mri_path,
    seg_path,
    output_dir,
    mouse_id=mouse_id,
    segment_format=segment_format,
    force=force,
    ignore_list=ignore_list,
)

# %%

# %%



"""
Command line script to install notebooks
"""

import argparse

from .. import util


def parse_args():
    parser = argparse.ArgumentParser(description="Install notebooks into a specified directory")
    parser.add_argument("output", nargs="?", help="path to the output file")
    parser.add_argument(
        "-f",
        "--force",
        action="store_true",
        default=False,
        help="overwrite files",
    )
    return parser.parse_args()


def main():
    args = parse_args()
    util.install_notebooks(args.output, force=args.force)
    return 0


if __name__ == "__main__":
    main()



# ---
# jupyter:
#   jupytext:
#     formats: py:percent
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.16.3
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

import datetime
import logging

# %%
import os
from pathlib import Path

import numpy as np
import pandas as pd
import SimpleITK as sitk
import trimesh
from aind_anatomical_utils import coordinate_systems as cs
from aind_mri_utils.arc_angles import arc_angles_to_affine
from aind_mri_utils.chemical_shift import (
    chemical_shift_transform,
    compute_chemical_shift,
)
from aind_mri_utils.file_io.obj_files import get_vertices_and_faces
from aind_mri_utils.file_io.simpleitk import load_sitk_transform
from aind_mri_utils.implant import make_hole_seg_dict
from aind_mri_utils.meshes import (
    apply_transform_to_trimesh,
    load_newscale_trimesh,
)
from aind_mri_utils.reticle_calibrations import (
    combine_parallax_and_manual_calibrations,
    find_probe_angle,
    fit_rotation_params_from_parallax,
)
from aind_mri_utils.rotations import (
    apply_rotate_translate,
    compose_transforms,
    invert_rotate_translate,
)
from scipy.spatial.transform import Rotation

# %%
# %matplotlib ipympl

# %%
# Set the log verbosity to get debug statements
logging.basicConfig(format="%(message)s", level=logging.DEBUG)
# %%
# File Paths
mouse = "786866"
reticle_used = "H"
target_structures = ["PL", "CLA", "MD", "CA1", "VM", "BLA", "RSP"]

WHOAMI = "Galen"

if WHOAMI == "Galen":
    base_path = Path("/mnt/aind1-vast/scratch")
elif WHOAMI == "Yoni":
    base_path = Path(r"Y:/")
else:
    raise ValueError("Who are you again?")

# File Paths
# Image and image annotations.
annotations_path = base_path / "ephys/persist/data/MRI/processed/{}".format(mouse)
image_path = annotations_path / "{}_100.nii.gz".format(mouse)
labels_path = annotations_path / "{}_HeadframeHoles.seg.nrrd".format(mouse)
brain_mask_path = annotations_path / "{}_manual_skull_strip.nrrd".format(mouse)
image_transform_file = annotations_path / "com_plane.h5"
structure_mask_path = annotations_path / "Masks"
structure_files = {structure: structure_mask_path / f"{mouse}{structure}Mask.obj" for structure in target_structures}
brain_mesh_path = structure_mask_path / "{}_manual_skull_strip.obj".format(mouse)

# Implant annotation
# Note that this can be different than the image annotation,
# this is in the event that an instion is planned with data from multiple scans
# (see 750107 for example).
implant_annoation_path = base_path / "ephys/persist/data/MRI/processed/{}".format(mouse)
headframe_transform_file = implant_annoation_path / "com_plane.h5"
implant_file = implant_annoation_path / "{}_ImplantHoles.seg.nrrd".format(mouse)
implant_mesh_file = implant_annoation_path / "{}_ImplantHoles.obj".format(mouse)
implant_fit_transform_file = implant_annoation_path / "{}_implant_fit.h5".format(mouse)


# OBJ files
model_path = base_path / "ephys/persist/data/MRI/HeadframeModels"
hole_model_path = model_path / "HoleOBJs"
modified_probe_mesh_file = model_path / "modified_probe_holder.obj"


probe_model_files = {
    "2.1-alpha": model_path / "Centered_Newscale_2pt0.obj",
    "2.1": model_path / "dovetailtweezer_oneShank_centered_corrected.obj",
    "quadbase": model_path / "Quadbase_customHolder_centeredOnShank0.obj",
    "2.4": model_path / "dovetailwtweezer_fourShank_centeredOnShank0.obj",
    "pipette": model_path / "injection_pipette.obj",
}

headframe_file = model_path / "TenRunHeadframe.obj"
holes_file = model_path / "OneOff_HolesOnly.obj"
cone_file = model_path / "TacoForBehavior" / "0160-200-72_X06.obj"
well_file = model_path / "WHC_Well" / "0274-400-07_X02.obj"
implant_model_file = model_path / "0283-300-04.obj"

calibration_path = base_path / "ephys/persist/data/probe_calibrations/CSVCalibrations/"
calibration_file = calibration_path / "calibration_info_np2_2025_03_18T08_39_00.xlsx"
parallax_calibration_dir = calibration_path / "log_20250303_122136"
iso_time = datetime.datetime.now().astimezone().strftime("%Y-%m-%dT%H%M%S%z")
plan_save_path = annotations_path / f"{mouse}_InsertionPlan_{iso_time}.csv"

# %%
# Reticle offsets and rotations
reticle_offsets = {"H": np.array([0.076, 0.062, 0.311])}
reticle_rotations = {"H": 0}

# %%
image_R, image_t, image_c = load_sitk_transform(str(image_transform_file))
headframe_R, headframe_t, headframe_c = load_sitk_transform(str(headframe_transform_file))

image = sitk.ReadImage(str(image_path))

# Load the headframe
headframe, headframe_faces = get_vertices_and_faces(headframe_file)
headframe_lps = cs.convert_coordinate_system(headframe, "ASR", "LPS")  # Preserves shape!

# Load the headframe
cone, cone_faces = get_vertices_and_faces(cone_file)
cone_lps = cs.convert_coordinate_system(cone, "ASR", "LPS")  # Preserves shape!

well, well_faces = get_vertices_and_faces(well_file)
well_lps = cs.convert_coordinate_system(well, "ASR", "LPS")  # Preserves shape!

implant_model, implant_faces = get_vertices_and_faces(implant_model_file)
implant_model_lps = cs.convert_coordinate_system(implant_model, "ASR", "LPS")  # Preserves shape!

# Load the brain mask
mask = sitk.ReadImage(str(brain_mask_path))
idxx = np.where(sitk.GetArrayViewFromImage(mask))
idx = np.vstack((idxx[2], idxx[1], idxx[0])).T
brain_pos = np.zeros(idx.shape)
brain_pos = np.vstack([mask.TransformIndexToPhysicalPoint(idx[ii, :].tolist()) for ii in range(idx.shape[0])])
brain_pos = brain_pos[np.arange(0, brain_pos.shape[0], brain_pos.shape[0] // 1000)]

# Load the brain mesh
brain_mesh = trimesh.load(
    str(brain_mesh_path),
    force="Mesh",
)

implant_seg_vol = sitk.ReadImage(str(implant_file))

probe_models = {k: load_newscale_trimesh(v, 0) for k, v in probe_model_files.items()}

# %%

# Get the trimesh objects for each hole.
# These are made using blender from the cad file
hole_files = [x for x in os.listdir(hole_model_path) if ".obj" in x and "Hole" in x]
hole_dict = {}
for i, flname in enumerate(hole_files):
    hole_num = int(flname.split("Hole")[-1].split(".")[0])
    hole_dict[hole_num] = trimesh.load(os.path.join(hole_model_path, flname))
    hole_dict[hole_num].vertices = cs.convert_coordinate_system(
        hole_dict[hole_num].vertices, "ASR", "LPS"
    )  # Preserves shape!

# Get the lower face, store with key -1
hole_dict[-1] = trimesh.load(os.path.join(hole_model_path, "LowerFace.obj"))
hole_dict[-1].vertices = cs.convert_coordinate_system(hole_dict[-1].vertices, "ASR", "LPS")  # Preserves shape!


# %%
model_implant_targets = {}
for i, hole_id in enumerate(hole_dict.keys()):
    if hole_id < 0:
        continue
    model_implant_targets[hole_id] = hole_dict[hole_id].centroid

# %%
# If implant has holes that are segmented.

implant_targets_by_hole = make_hole_seg_dict(implant_seg_vol, fun=lambda x: np.mean(x, axis=0))
implant_names = list(implant_targets_by_hole.keys())
implant_targets = np.vstack(list(implant_targets_by_hole.values()))

# %%
chem_shift_pt_R, chem_shift_pt_t = chemical_shift_transform(compute_chemical_shift(image, ppm=3.7))
chem_shift_image_R, chem_shift_image_t = invert_rotate_translate(chem_shift_pt_R, chem_shift_pt_t)
chem_image_R, chem_image_t = compose_transforms(chem_shift_image_R, chem_shift_image_t, image_R, image_t)

# %%

transformed_brain = apply_rotate_translate(brain_pos, *invert_rotate_translate(chem_image_R, chem_image_t))
transformed_brain_mesh = apply_transform_to_trimesh(
    brain_mesh.copy(), *invert_rotate_translate(chem_image_R, chem_image_t)
)
transformed_implant_targets = apply_rotate_translate(
    implant_targets, *invert_rotate_translate(headframe_R, headframe_t)
)

# %%
# Find calibrated probes

# Read the calibrations
reticle_offset = reticle_offsets[reticle_used]
reticle_rotation = reticle_rotations[reticle_used]
if calibration_file is None:
    cal_by_probe_combined, R_reticle_to_bregma = fit_rotation_params_from_parallax(
        parallax_calibration_dir,
        reticle_offset,
        reticle_rotation,
        find_scaling=True,
    )
    global_offset = reticle_offset
else:
    cal_by_probe_combined, R_reticle_to_bregma, global_offset = combine_parallax_and_manual_calibrations(
        manual_calibration_files=calibration_file,
        parallax_directories=parallax_calibration_dir,
    )

probes_used = list(cal_by_probe_combined.keys())

# %%
probe_to_target_mapping = {
    "PL": 45883,
    "CLA": 46110,
    "MD": 46116,
    "VM": 46100,
    "CA1": 46113,
    "BLA": 46122,
    "RSP": 50209,
}

# %%
arcs = {
    "a": 18,
    "b": 2,
    "c": -12.745,
    "d": -30,
}
probe_by_struct = {
    "PL": "quadbase",
    "CLA": "2.1-alpha",
    "MD": "2.4",
    "CA1": "2.1-alpha",
    "VM": "2.1-alpha",
    "BLA": "2.1",
    "RSP": "2.1",
}
arc_by_struct = {
    "PL": "a",
    "CLA": "a",
    "MD": "c",
    "CA1": "b",
    "VM": "b",
    "BLA": "d",
    "RSP": "d",
}
arc_ap_by_struct = {k: arcs[v] for k, v in arc_by_struct.items()}
slider_ml_by_struct = {
    "PL": -20,
    "CLA": 29.5,
    "MD": -23,
    "CA1": 25,
    "VM": -19,
    "BLA": 3,
    "RSP": 19,
}
spin_by_struct = {
    "PL": -45,
    "CLA": 145,
    "MD": 90,
    "CA1": 90,
    "VM": -90,
    "BLA": 110,
    "RSP": 135,
}

# This should be the x offset and the inverse of the y offset
offsets_LP_by_struct = {
    "PL": [-0.25, 0.25],
    "CLA": [0.0, -0.4],
    "MD": [0, -0.35],
    "CA1": [-0.1, 0.4],
    "VM": [0, 0.2],
    "BLA": [0, 0.1],
    "RSP": [0, 0],
}
offsets_LP_arr_by_struct = {k: np.array(v) for k, v in offsets_LP_by_struct.items()}

target_depth = np.array([2.4, 4.75, 4, 5.8, 4.9, 5.75, 2])  # Guesses, check
depth_by_struct = {
    "PL": 2.4,
    "CLA": 5.5,
    "MD": 4,
    "CA1": 6.5,
    "VM": 4.9,
    "BLA": 5.75,
    "RSP": 2,
}

hole_by_struct = {
    "PL": 1,
    "CLA": 2,
    "MD": 6,
    "CA1": 4,
    "VM": 8,
    "BLA": 10,
    "RSP": 5,
}

insertion_trim_coordinates_struct = {}

# %%
implant_R, implant_t, implant_c = load_sitk_transform(implant_fit_transform_file)

S = trimesh.Scene()
transformed_implant_targets = {}

for i, key in enumerate(model_implant_targets.keys()):
    implant_tgt = model_implant_targets[key]
    implant_tgt = apply_rotate_translate(implant_tgt, *invert_rotate_translate(implant_R, implant_t))
    implant_tgt = apply_rotate_translate(implant_tgt, *invert_rotate_translate(headframe_R, headframe_t))
    transformed_implant_targets[key] = implant_tgt

vertices = apply_rotate_translate(implant_model_lps, *invert_rotate_translate(implant_R, implant_t))
vertices = apply_rotate_translate(vertices, *invert_rotate_translate(headframe_R, headframe_t))
implant_mesh = trimesh.Trimesh(vertices=vertices, faces=implant_faces[0])


# holeCM = trimesh.collision.CollisionManager()
implantCM = trimesh.collision.CollisionManager()
probeCM = trimesh.collision.CollisionManager()
coneCM = trimesh.collision.CollisionManager()
wellCM = trimesh.collision.CollisionManager()

from_calibration = True
ras_to_lps = np.diag([-1, -1, 1])
final_target_by_struct = {}
for i, structure in enumerate(target_structures):
    if structure not in [
        # "PL",
        "CLA",
        "MD",
        "CA1",
        "VM",
        "BLA",
        "RSP",
    ]:  # target_structures:
        continue

    structureCM = trimesh.collision.CollisionManager()

    probe_type = probe_by_struct[structure]
    ap_angle = arcs[arc_by_struct[structure]]
    ml_angle = slider_ml_by_struct[structure]
    LP_offset = offsets_LP_arr_by_struct[structure]
    depth = depth_by_struct[structure]
    probe_model = probe_models[probe_type].copy()
    hole_number = hole_by_struct[structure]
    spin = spin_by_struct[structure]

    # Generate a single random color for both probe and structure
    this_color = trimesh.visual.random_color()

    # Assign the same color to the probe
    probe_model.visual.face_colors = this_color

    # Apply transformations to the probe
    implant_target = transformed_implant_targets[hole_number]

    this_pt = trimesh.creation.uv_sphere(radius=0.25)
    this_pt.apply_translation(implant_target)
    this_pt.visual.vertex_colors = [255, 0, 255, 255]
    S.add_geometry(this_pt)

    offset = np.zeros(3)
    offset[:2] = LP_offset
    adjusted_insertion_pt = implant_target + offset

    if from_calibration:
        this_probe = probe_to_target_mapping[structure]
        this_affine, this_translation, this_scaling = cal_by_probe_combined[this_probe]
        insertion_vector = ras_to_lps @ np.linalg.inv(this_affine) @ np.array([0, 0, depth])
        insertion_trims = insertion_trim_coordinates_struct.get(structure, np.array([0, 0, 0]))
        combined_bregma_vector = adjusted_insertion_pt + insertion_vector + insertion_trims
        scaling_inv = np.diag(1 / this_scaling)
        rigid_affine = scaling_inv @ this_affine
        this_ap, this_ml = find_probe_angle(rigid_affine)

        R_probe_mesh = arc_angles_to_affine(this_ap, this_ml, spin)

        probe_model = apply_transform_to_trimesh(probe_model, R_probe_mesh, combined_bregma_vector)
    else:
        R_probe_mesh = arc_angles_to_affine(ap_angle, ml_angle, spin)
        insertion_vector = R_probe_mesh @ np.array([0, 0, -depth])
        combined_bregma_vector = adjusted_insertion_pt + insertion_vector
        probe_model = apply_transform_to_trimesh(probe_model, R_probe_mesh, combined_bregma_vector)

    final_target_by_struct[structure] = ras_to_lps @ combined_bregma_vector

    S.add_geometry(probe_model)
    probeCM.add_object(structure, probe_model)
    implantCM.add_object(structure, probe_model)
    coneCM.add_object(structure, probe_model)
    wellCM.add_object(structure, probe_model)

    structureCM.add_object("probe", probe_model)

    # Load and transform the target structure
    this_target_mesh = trimesh.load(
        str(structure_files[structure]),
        force="Mesh",
    )
    trimesh.repair.fix_normals(this_target_mesh)
    trimesh.repair.fix_inversion(this_target_mesh)

    vertices = this_target_mesh.vertices
    vertices = apply_rotate_translate(vertices, *invert_rotate_translate(chem_image_R, chem_image_t))
    this_target_mesh.vertices = vertices

    # Assign the same color to the structure
    this_target_mesh.visual.face_colors = this_color
    this_target_mesh.visual.vertex_colors = this_color
    this_target_mesh.visual.material.main_color[:] = this_color

    # Add structure to the scene and collision manager
    S.add_geometry(this_target_mesh)
    structureCM.add_object("structure", this_target_mesh)

    # Check collisions
    if structureCM.in_collision_internal(False, False):
        print(f"Probe for {structure} is a hit :)")
    else:
        print(f"Probe for {structure} is a miss! :(")
    print(i)


# S.add_geometry(transformed_brain_mesh)
headframe_mesh = trimesh.Trimesh(vertices=headframe_lps, faces=headframe_faces[0])
cone_mesh = trimesh.Trimesh(vertices=cone_lps, faces=cone_faces[0])
coneCM.add_object("cone", headframe_mesh)

well_mesh = trimesh.Trimesh(vertices=well_lps, faces=well_faces[0])
wellCM.add_object("well", well_mesh)

implantCM.add_object("implant", implant_mesh)

# Optionally assign unique colors to headframe, cone, and well if desired:
headframe_color = trimesh.visual.random_color()
cone_color = trimesh.visual.random_color()
well_color = trimesh.visual.random_color()

headframe_mesh.visual.face_colors = headframe_color
headframe_mesh.vertices
cone_mesh.visual.face_colors = cone_color
well_mesh.visual.face_colors = well_color

S.add_geometry(headframe_mesh)
# S.add_geometry(cone_mesh)
S.add_geometry(well_mesh)

probe_fail, fail_names = probeCM.in_collision_internal(return_names=True)
if probe_fail:
    print("Probes are colliding :(")
    print(f"Problems: {list(fail_names)}")
else:
    print("Probes are clear! :)")

    if coneCM.in_collision_internal(False, False):
        print("Probes are hitting cone! :(")
    else:
        print("Probes are clearing cone :)")

    if wellCM.in_collision_internal(False, False):
        print("Probes are hitting well! :(")
    else:
        print("Probes are clearing well :)")

probe_fail, fail_names_2 = implantCM.in_collision_internal(return_names=True)
if probe_fail:
    print("Probes are striking implant! :(")
    print(f"problems: {list(fail_names_2)}")
else:
    print("Probes clear implant! :)")
S.add_geometry(implant_mesh)
S.show(viewer="gl")

# %%
mouse_to_rig_ap = 14
ap_angle_rig_by_struct = {k: v + mouse_to_rig_ap for k, v in arc_ap_by_struct.items()}
bregma_dims = ["ML", "AP", "DV"]
cols = {}
for structure in target_structures:
    if structure not in final_target_by_struct:
        continue
    cols.setdefault("Structure", []).append(structure)
    cols.setdefault("Probe type", []).append(probe_by_struct[structure])
    cols.setdefault("Arc", []).append(arc_by_struct[structure])
    cols.setdefault("AP angle", []).append(ap_angle_rig_by_struct[structure])
    cols.setdefault("ML angle", []).append(slider_ml_by_struct[structure])
    cols.setdefault("Spin", []).append(spin_by_struct[structure])
    cols.setdefault("Hole", []).append("Hole {}".format(hole_by_struct[structure]))
    cols.setdefault("Approx. depth", []).append(depth_by_struct[structure])

    target = final_target_by_struct[structure]
    for dim, dim_val in zip(bregma_dims, target):
        cols.setdefault(dim, []).append(np.round(dim_val, 3))

plan_df = pd.DataFrame.from_dict(cols).set_index("Structure").sort_values(by=["Arc"])
if plan_save_path is not None:
    plan_df.to_csv(plan_save_path)
plan_df
# %%
tgt_structure = "CLA"
this_probe = probe_to_target_mapping[tgt_structure]
this_affine = cal_by_probe_combined[this_probe][0]
this_ap, this_ml = find_probe_angle(this_affine)
R_probe_mesh = arc_angles_to_affine(this_ap, this_ml)
R_probe_mesh[:3, :3]

# %%
name = []
ML = []
AP = []
DV = []
source = []
for i, structure in enumerate(target_structures):
    probe_type = probe_by_struct[structure]
    ap_angle = arcs[arc_by_struct[structure]]
    ml_angle = slider_ml_by_struct[structure]
    LP_offset = offsets_LP_arr_by_struct[structure]
    depth = depth_by_struct[structure]
    probe_model = probe_models[probe_type].copy()
    hole_number = hole_by_struct[structure]
    spin = spin_by_struct[structure]

    print(structure)
    print(f"AP: {ap_angle + 14}; ML: {ml_angle}; Spin: {spin}")
    print(f"Hole: {hole_number}")
    hole_coord = transformed_implant_targets[hole_number]
    offset = np.zeros(3)
    offset[:2] = LP_offset
    adjusted_insertion_pt = implant_target + offset

    hole_coord_ras = ras_to_lps @ hole_coord
    hole_ml, hole_ap, hole_dv = hole_coord_ras
    name.append(hole_number)
    print(f"Hole Target: ML: {hole_ml}  AP: {hole_ap} DV: {hole_dv}")
    ML.append(hole_ml)
    AP.append(hole_ap)
    DV.append(hole_dv)
    source.append("insertion plan")
    print(f"Distance past target: {depth}")
    print(f"Needs probe: {probe_type}")
    print("\n")
this_affine.T @ np.array([[-1, 0, 0], [0, 1, 0], [0, 0, -1]])

# %%

R = Rotation.from_matrix(this_affine.T @ np.array([[-1, 0, 0], [0, 1, 0], [0, 0, -1]])).as_euler("xyz")
R

# %%
R = Rotation.from_matrix(R_probe_mesh[:3, :3]).as_euler("xyz")
R

# %%



# %%
from pathlib import Path

import numpy as np
import pandas as pd
import SimpleITK as sitk
import trimesh
from aind_anatomical_utils import coordinate_systems as cs
from aind_anatomical_utils import slicer as sf
from aind_mri_utils import rotations as rot
from aind_mri_utils.chemical_shift import (
    chemical_shift_transform,
    compute_chemical_shift,
)
from aind_mri_utils.file_io import simpleitk as mr_sitk
from aind_mri_utils.file_io.obj_files import get_vertices_and_faces
from aind_mri_utils.meshes import load_newscale_trimesh
from aind_mri_utils.planning import (
    candidate_insertions,
    compatible_insertion_pairs,
    find_other_compatible_insertions,
    get_implant_targets,
    make_scene_for_insertion,
)

# %%
mouse = "750105"
whoami = "galen"
if whoami == "galen":
    base_dir = Path("/mnt/aind1-vast/scratch/")
    base_save_dir = Path("/home/galen.lynch/")
elif whoami == "yoni":
    base_dir = Path("Y:/")
    base_save_dir = Path("C:/Users/yoni.browning/OneDrive - Allen Institute/Desktop/")
else:
    raise ValueError("Who are you again?")

headframe_model_dir = base_dir / "ephys/persist/data/MRI/HeadframeModels/"
probe_model_file = (
    headframe_model_dir / "dovetailtweezer_oneShank_centered_corrected.obj"
)  # "modified_probe_holder.obj"
annotations_path = base_dir / "ephys/persist/data/MRI/processed/{}".format(mouse)

headframe_path = headframe_model_dir / "TenRunHeadframe.obj"
holes_path = headframe_model_dir / "OneOff_HolesOnly.obj"


implant_holes_path = str(annotations_path / "{}_ImplantHoles.seg.nrrd".format(mouse))

image_path = str(annotations_path / "{}_100.nii.gz".format(mouse))  # '_100.nii.gz'))
labels_path = str(annotations_path / "{}_HeadframeHoles.seg.nrrd".format(mouse))  # 'Segmentation.seg.nrrd')#
brain_mask_path = str(annotations_path / ("{}_auto_skull_strip.nrrd".format(mouse)))
manual_annotation_path = str(annotations_path / (f"{mouse}_ManualAnnotations.fcsv"))
cone_path = base_dir / "ephys/persist/Software/PinpointBuilds/WavefrontFiles/Cone_0160-200-53.obj"

uw_yoni_annotation_path = annotations_path / f"targets-{mouse}-transformed.fcsv"

newscale_file_name = headframe_path / "Centered_Newscale_2pt0.obj"
#


calibration_filename = "calibration_info_np2_2024_04_22T11_15_00.xlsx"
calibration_dir = base_dir / "ephys/persist/data/probe_calibrations/CSVCalibrations/"
calibration_file = calibration_dir / calibration_filename
measured_hole_centers = annotations_path / "measured_hole_centers_conflict.xlsx"


# manual_hole_centers_file = annotations_path / 'hole_centers.mrk.json'

transformed_targets_save_path = annotations_path / (f"{mouse}_TransformedTargets.csv")
test_probe_translation_save_path = str(base_save_dir / "test_probe_translation.h5")
transform_filename = str(annotations_path / (mouse + "_com_plane.h5"))

# %%
target_structures = None  # ["CCant", "CCpst", "AntComMid", "GenFacCran2"]

# %%
image = sitk.ReadImage(image_path)
# Read points
manual_annotation = sf.read_slicer_fcsv(manual_annotation_path)

# Load the headframe
headframe, headframe_faces = get_vertices_and_faces(headframe_path)
headframe_lps = cs.convert_coordinate_system(headframe, "ASR", "LPS")  # Preserves shape!

# Load the computed transform
trans = mr_sitk.load_sitk_transform(transform_filename, homogeneous=True, invert=True)[0]

cone = trimesh.load_mesh(cone_path)
cone.vertices = cs.convert_coordinate_system(cone.vertices, "ASR", "LPS")

probe_mesh = load_newscale_trimesh(probe_model_file, move_down=0.5)

# Get chemical shift from MRI image.
# Defaults are standard UW scans- set params for anything else.
chem_shift = compute_chemical_shift(image)
chem_shift_trans = chemical_shift_transform(chem_shift, readout="HF")
# -

# List targeted locations
if target_structures is None:
    target_structures = list(manual_annotation.keys())
preferred_pts = {k: manual_annotation[k] for k in target_structures}

hmg_pts = rot.prepare_data_for_homogeneous_transform(np.array(tuple(preferred_pts.values())))
chem_shift_annotation = hmg_pts @ chem_shift_trans.T @ trans.T
transformed_annotation = rot.extract_data_for_homogeneous_transform(chem_shift_annotation)
target_names = tuple(preferred_pts.keys())


# %%

implant_vol = sitk.ReadImage(implant_holes_path)

implant_targets, implant_names = get_implant_targets(implant_vol)


# Visualize Holes, list locations

transformed_implant_homog = rot.prepare_data_for_homogeneous_transform(implant_targets) @ trans.T
transformed_implant = rot.extract_data_for_homogeneous_transform(transformed_implant_homog)


# %%
dim_names = ["ML (mm)", "AP (mm)", "DV (mm)"]
transformed_annotation_ras = np.array([-1, -1, 1]) * transformed_annotation
target_df = pd.DataFrame(
    data={
        "point": target_names,
        **{d: transformed_annotation_ras[:, i] for i, d in enumerate(dim_names)},
    }
)
sp = np.argsort(implant_names)
implant_names_sorted = np.array(implant_names)[sp]
transformed_implant_sorted_ras = np.array([-1, -1, 1]) * transformed_implant[sp, :]
implant_df = pd.DataFrame(
    data={
        "point": [f"Hole {n}" for n in implant_names_sorted],
        **{d: transformed_implant_sorted_ras[:, i] for i, d in enumerate(dim_names)},
    }
)
df_joined = pd.concat((target_df, implant_df), ignore_index=True)
df_joined.to_csv(transformed_targets_save_path, index=False)
# %%
for i, n in enumerate(implant_names_sorted):
    print(f"{n}: {transformed_implant_sorted_ras[i]}")
# %%
df = candidate_insertions(
    transformed_annotation,
    transformed_implant,
    target_names,
    implant_names,
)
compat_matrix = compatible_insertion_pairs(df)


# %%
seed_insertions = []
bad_holes = [0, 1, 2, 7, 9, 10]
bad_mask = np.isin(df.hole.to_numpy(), bad_holes)
target_mask = df.target.to_numpy() == "CCant"
keep_mask = np.logical_and(np.logical_not(bad_mask), target_mask)
consider_ndxs = np.nonzero(keep_mask)[0]
target_ndxs = np.nonzero(df.target.to_numpy() == "CCant")[0]
print(df.iloc[consider_ndxs])

# %%
nonbad_ndxs = np.nonzero(np.logical_not(bad_mask))[0]
seed_insertions = [48]
compatible_insertions = find_other_compatible_insertions(compat_matrix, nonbad_ndxs, seed_insertions)
df.loc[np.concatenate([seed_insertions, compatible_insertions])]


# %%
headframe_mesh = trimesh.Trimesh()
headframe_mesh.vertices = headframe_lps
headframe_mesh.faces = headframe_faces

S = make_scene_for_insertion(
    headframe_mesh,
    cone,
    transformed_implant,
    transformed_annotation,
    seed_insertions,
    df,
    probe_mesh,
)

S.show()

# %%



"""
Tests for util.py module
"""

from pathlib import Path
from unittest.mock import patch

import pytest

from aind_mri_targeting.util import check_output_path, err_if_files_exist, install_notebooks


class TestInstallNotebooks:
    """Test cases for install_notebooks function"""

    def test_install_notebooks_default_output(self, tmp_path, monkeypatch):
        """Test install_notebooks with default output (cwd)"""
        # Create a temporary notebooks directory structure
        notebooks_dir = tmp_path / "notebooks"
        notebooks_dir.mkdir()

        # Create test notebook files
        test_file1 = notebooks_dir / "test_notebook1.py"
        test_file2 = notebooks_dir / "test_notebook2.py"
        test_file1.write_text("# Test notebook 1")
        test_file2.write_text("# Test notebook 2")

        # Create a temporary working directory
        work_dir = tmp_path / "work"
        work_dir.mkdir()

        # Change to the working directory
        monkeypatch.chdir(work_dir)

        # Mock the Path resolution to point to our test notebooks
        with patch("aind_mri_targeting.util.Path") as mock_path:
            notebooks_output = work_dir / "notebooks"
            mock_path.return_value.__truediv__.return_value = notebooks_output
            mock_resolve = mock_path.return_value.resolve.return_value
            mock_resolve.parent.parent.parent = tmp_path

            # Mock glob to return our test files
            with patch("aind_mri_targeting.util.glob.glob") as mock_glob:
                mock_glob.return_value = [str(test_file1), str(test_file2)]

                install_notebooks()

                # Check that the notebooks directory was created
                output_notebooks_dir = work_dir / "notebooks"
                assert output_notebooks_dir.exists()

                # Check that files were copied
                assert (output_notebooks_dir / "test_notebook1.py").exists()
                assert (output_notebooks_dir / "test_notebook2.py").exists()

    def test_install_notebooks_custom_output(self, tmp_path):
        """Test install_notebooks with custom output directory"""
        # Create a temporary notebooks directory structure
        notebooks_dir = tmp_path / "notebooks"
        notebooks_dir.mkdir()

        # Create test notebook files
        test_file = notebooks_dir / "test_notebook.py"
        test_file.write_text("# Test notebook")

        # Create custom output directory
        output_dir = tmp_path / "custom_output"
        output_dir.mkdir()

        # Mock the Path resolution and glob
        with patch("aind_mri_targeting.util.Path") as mock_path:
            notebooks_output = output_dir / "notebooks"
            mock_path.return_value.__truediv__.return_value = notebooks_output
            mock_resolve = mock_path.return_value.resolve.return_value
            mock_resolve.parent.parent.parent = tmp_path

            with patch("aind_mri_targeting.util.glob.glob") as mock_glob:
                mock_glob.return_value = [str(test_file)]

                install_notebooks(output=str(output_dir))

                # Check notebooks directory was created in custom location
                output_notebooks_dir = output_dir / "notebooks"
                assert output_notebooks_dir.exists()
                assert (output_notebooks_dir / "test_notebook.py").exists()

    def test_install_notebooks_skip_existing_files(self, tmp_path):
        """Test that existing files are skipped when force=False"""
        # Create notebooks directory and test file
        notebooks_dir = tmp_path / "notebooks"
        notebooks_dir.mkdir()
        test_file = notebooks_dir / "existing_notebook.py"
        test_file.write_text("# Original content")

        # Create output directory with existing file
        output_dir = tmp_path / "output"
        output_notebooks_dir = output_dir / "notebooks"
        output_notebooks_dir.mkdir(parents=True)
        existing_output_file = output_notebooks_dir / "existing_notebook.py"
        existing_output_file.write_text("# Existing content")

        # Mock Path and glob
        with patch("aind_mri_targeting.util.Path") as mock_path:
            mock_path.return_value.__truediv__.return_value = output_notebooks_dir
            mock_resolve = mock_path.return_value.resolve.return_value
            mock_resolve.parent.parent.parent = tmp_path
            mock_path.return_value.name = "existing_notebook.py"

            with patch("aind_mri_targeting.util.glob.glob") as mock_glob:
                mock_glob.return_value = [str(test_file)]

                with patch("builtins.print") as mock_print:
                    install_notebooks(output=str(output_dir), force=False)

                    # Should print skip message
                    expected_msg = "Skipping existing_notebook.py"
                    mock_print.assert_called_with(expected_msg)

                    # File content should remain unchanged
                    expected_content = "# Existing content"
                    assert existing_output_file.read_text() == expected_content

    def test_install_notebooks_force_overwrite(self, tmp_path):
        """Test that existing files are overwritten when force=True"""
        # Create notebooks directory and test file
        notebooks_dir = tmp_path / "notebooks"
        notebooks_dir.mkdir()
        test_file = notebooks_dir / "notebook.py"
        test_file.write_text("# New content")

        # Create output directory with existing file
        output_dir = tmp_path / "output"
        output_notebooks_dir = output_dir / "notebooks"
        output_notebooks_dir.mkdir(parents=True)
        existing_output_file = output_notebooks_dir / "notebook.py"
        existing_output_file.write_text("# Old content")

        # Mock shutil.copy2 to simulate copying
        with patch("aind_mri_targeting.util.shutil.copy2") as mock_copy:
            with patch("aind_mri_targeting.util.Path") as mock_path:
                mock_path.return_value.__truediv__.return_value = output_notebooks_dir
                mock_resolve = mock_path.return_value.resolve.return_value
                mock_resolve.parent.parent.parent = tmp_path
                mock_path.return_value.name = "notebook.py"

                with patch("aind_mri_targeting.util.glob.glob") as mock_glob:
                    mock_glob.return_value = [str(test_file)]

                    with patch("builtins.print") as mock_print:
                        install_notebooks(output=str(output_dir), force=True)

                        # Should print copy message
                        mock_print.assert_called_with("Copying notebook.py")

                        # Should call copy2
                        mock_copy.assert_called_once()

    def test_install_notebooks_creates_output_directory(self, tmp_path):
        """Test that output directory is created if it doesn't exist"""
        # Create notebooks directory
        notebooks_dir = tmp_path / "notebooks"
        notebooks_dir.mkdir()
        test_file = notebooks_dir / "test.py"
        test_file.write_text("# Test")

        # Output directory that doesn't exist yet
        output_dir = tmp_path / "new_output"

        with patch("aind_mri_targeting.util.Path") as mock_path:
            mock_path.return_value.__truediv__.return_value = output_dir / "notebooks"
            mock_resolve = mock_path.return_value.resolve.return_value
            mock_resolve.parent.parent.parent = tmp_path

            with patch("aind_mri_targeting.util.glob.glob") as mock_glob:
                mock_glob.return_value = [str(test_file)]

                install_notebooks(output=str(output_dir))

                # Directory should be created
                assert (output_dir / "notebooks").exists()


class TestCheckOutputPath:
    """Test cases for check_output_path function"""

    def test_check_output_path_valid_directory(self, tmp_path):
        """Test check_output_path with a valid directory"""
        result = check_output_path(str(tmp_path))
        assert isinstance(result, Path)
        assert result == Path(tmp_path)

    def test_check_output_path_none_uses_cwd(self):
        """Test check_output_path with None uses current working directory"""
        with patch("aind_mri_targeting.util.os.getcwd") as mock_getcwd:
            mock_getcwd.return_value = "/fake/cwd"
            with patch("aind_mri_targeting.util.os.path.isdir") as mock_isdir:
                mock_isdir.return_value = True

                result = check_output_path(None)

                assert isinstance(result, Path)
                assert str(result) == "/fake/cwd"
                mock_getcwd.assert_called_once()

    def test_check_output_path_invalid_directory(self, tmp_path):
        """Test check_output_path raises NotADirectoryError for invalid path"""
        invalid_path = tmp_path / "nonexistent"

        with pytest.raises(NotADirectoryError) as exc_info:
            check_output_path(str(invalid_path))

        assert "is not a directory" in str(exc_info.value)
        assert str(invalid_path) in str(exc_info.value)

    def test_check_output_path_file_not_directory(self, tmp_path):
        """Test check_output_path raises error when path is a file"""
        file_path = tmp_path / "test_file.txt"
        file_path.write_text("test content")

        with pytest.raises(NotADirectoryError) as exc_info:
            check_output_path(str(file_path))

        assert "is not a directory" in str(exc_info.value)


class TestErrIfFilesExist:
    """Test cases for err_if_files_exist function"""

    def test_err_if_files_exist_no_files(self):
        """Test err_if_files_exist with non-existent files"""
        non_existent_files = ["/fake/path1.txt", "/fake/path2.txt"]

        # Should not raise any exception
        err_if_files_exist(non_existent_files)

    def test_err_if_files_exist_empty_list(self):
        """Test err_if_files_exist with empty file list"""
        # Should not raise any exception
        err_if_files_exist([])

    def test_err_if_files_exist_file_exists(self, tmp_path):
        """Test err_if_files_exist raises FileExistsError when file exists"""
        existing_file = tmp_path / "existing_file.txt"
        existing_file.write_text("test content")

        with pytest.raises(FileExistsError) as exc_info:
            err_if_files_exist([str(existing_file)])

        assert "already exists" in str(exc_info.value)
        assert "Set force=True to overwrite" in str(exc_info.value)
        assert str(existing_file) in str(exc_info.value)

    def test_err_if_files_exist_multiple_files_one_exists(self, tmp_path):
        """Test err_if_files_exist with multiple files where one exists"""
        existing_file = tmp_path / "existing.txt"
        existing_file.write_text("content")
        non_existing_file = tmp_path / "non_existing.txt"

        files_to_check = [str(non_existing_file), str(existing_file)]

        with pytest.raises(FileExistsError) as exc_info:
            err_if_files_exist(files_to_check)

        assert str(existing_file) in str(exc_info.value)

    def test_err_if_files_exist_custom_error_message(self, tmp_path):
        """Test err_if_files_exist with custom error message"""
        existing_file = tmp_path / "test.txt"
        existing_file.write_text("content")

        custom_message = "Custom error: {} is already there!"

        with pytest.raises(FileExistsError) as exc_info:
            err_if_files_exist([str(existing_file)], custom_message)

        assert "Custom error:" in str(exc_info.value)
        assert "is already there!" in str(exc_info.value)
        assert str(existing_file) in str(exc_info.value)

    def test_err_if_files_exist_returns_none_on_success(self):
        """Test that err_if_files_exist returns None when no files exist"""
        result = err_if_files_exist(["/fake/file1.txt", "/fake/file2.txt"])
        assert result is None



# ---
# jupyter:
#   jupytext:
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.16.3
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# %%
from pathlib import Path

from aind_mri_targeting.implant_rotations import fit_implant_to_mri_from_files

# %%
# Paths
mouse_id = 771433
whoami = "yoni"
if whoami == "galen":
    scratchdir = Path("/mnt/aind1-vast/scratch/")
    base_save_dir = Path("/home/galen.lynch/")
elif whoami == "yoni":
    scratchdir = Path(r"Y:")
mri_folder = scratchdir / "ephys/persist/data/MRI"
processed_folder = mri_folder / "processed"
mouse_folder = processed_folder / str(mouse_id)

# required files
implant_annotations_file = mouse_folder / f"{mouse_id}_ImplantHoles.seg.nrrd"
hole_folder = mri_folder / "HeadframeModels/HoleOBJs"
save_name = str(mouse_folder)  # f"{mouse_id}_implant_fit.h5")

# optional flags
save_inverse = True  # save the inverse transformation matrix, which works on volumes
force = True

# %%
# Run the fitting
fit_implant_to_mri_from_files(
    implant_annotations_file,
    hole_folder,
    save_name=save_name,
    save_inverse=save_inverse,
    force=force,
    mouse_name=mouse_id,
)

# %%
save_name

# %%



# ---
# jupyter:
#   jupytext:
#     text_representation:
#       extension: .py
#       format_name: percent
#       format_version: '1.3'
#       jupytext_version: 1.16.3
#   kernelspec:
#     display_name: Python 3 (ipykernel)
#     language: python
#     name: python3
# ---

# %% [markdown]
# # Probe calibration and target transformation notebook
#
# This notebook is used to calibrate probes and transform targets from
# bregma-relative mm to manipulator coordinates
#
# # How to use this notebook
# 1. Set the mouse ID in the cell two below.
# 2. Set the path to the calibration files with the probe data.
# 3. Set the path to the target file.
# 4. Optionally set `fit_scale` to `True` if you want to fit the scale
# parameters as well. This is not recommended unless you have a good reason to
# do so. It does not guarantee that the error will be lower.
# 5. Run the next three cells to get the transformed targets, and see which
# targets are available
# 6. Configure the experiment by assigning each probe that you want to use to a
# target in the target file and specify the overshoot in µm. If you have
# targets that are not in the target file, you can specify them manually.
# 7. Run the next cell to fit the rotation parameters. If `verbose` is set to
# `True`, the mean and maximum error for each probe will be printed, as well as
# the predicted probe coordinates for each reticle coordinate with error for
# that coordinate.
# 8. Run the last cell to get the transformed targets in manipulator
# coordinates

# %%
import logging
from pathlib import Path

import numpy as np
import pandas as pd

# %matplotlib inline
from aind_mri_utils.reticle_calibrations import (
    debug_parallax_and_manual_calibrations,
    debug_parallax_calibration,
    transform_bregma_to_probe,
    transform_probe_to_bregma,
)

logger = logging.getLogger(__name__)

logging.basicConfig(format="%(message)s", level=logging.DEBUG)
# %%
# Set file paths and mouse ID here

# Calibration File with probe data
mouse_id = "786866"
reticle_used = "H"
basepath = Path("/mnt/vast/scratch/")
calibration_dir = basepath / "ephys/persist/data/probe_calibrations/CSVCalibrations/"
# Target file with transformed targets
target_dir = basepath / f"ephys/persist/data/MRI/processed/{mouse_id}/"
# Calibration directories to use for parallax, with the latter ones taking
# priority
parallax_calibration_directories = []
# Calibration files to use for manual calibration, with the latter ones taking
# priority. All manual calibrations will take priority over parallax
# calibrations
manual_calibration_filenames = ["calibration_info_np2_2025_03_18T08_39_00.xlsx"]
# List of probes to ignore manual calibrations from
probes_to_ignore_manual = []
target_file = None  # target_dir / f"{mouse_id}_TransformedTargets.csv"

parallax_calibration_paths = [calibration_dir / f for f in parallax_calibration_directories]
manual_calibration_paths = [
    basepath / "ephys/persist/data/MRI/processed/786866/calibration_info_348_sws1_2025_07_10T11_00_00.xlsx"
]  # [calibration_dir / f for f in manual_calibration_filenames]

# Whether to save the targets to a CSV file. If not None, the targets will be
# saved
save_path = None


# %%
# Reticle offsets and rotations
reticle_offsets = {"H": np.array([0.076, 0.062, 0.311])}
reticle_rotations = {"H": 0}


# %%
def _round_targets(target, probe_target):
    target_rnd = np.round(target, decimals=2)
    probe_target_and_overshoot_rnd = np.round(2000 * probe_target) / 2
    return target_rnd, probe_target_and_overshoot_rnd


# %%
if target_file:
    target_df = pd.read_csv(target_file)
    target_df = target_df.set_index("point")
else:
    target_df = None
# %% [markdown]
# ## Transformed targets
# print the transformed targets to see which targets are available
# %%
target_df

# %% [markdown]
# ## Configure experiment
# Assign each probe that you want to use to a target in the target file and
# specify the overshoot in µm. The format should be
# ```python
# targets_and_overshoots_by_probe = {
#     probe_id: (target_name, overshoot), # overshoot in µm
#     ...
# }
# ```
# Where each `probe_id` is the ID of a probe in the calibration file,
# `target_name` is the name of the target in the target file, and `overshoot`
# is the overshoot in µm.
#
# If you have targets that are not in the target file, you can specify them
# manually. The format should be
#
# ```python manual_bregma_targets_by_probe = { probe_id: [x, y, z], ...  } ```
# where `[x, y, z]` are the coordinates in mm.

# %%
# Set experiment configuration # here
#
# Names of targets in the target file and overshoots
# targets_and_overshoots_by_probe = {probe_id: (target_name, overshoot), ...}
# overshoot in µm
targets_and_overshoots_by_probe = {
    46110: ("Hole 1", 500),
    46100: ("Hole 2", 500),
}
# Targets in bregma-relative coordinates not in the target file
# manual_bregma_targets_by_probe = {probe_id: [x, y, z], ...}
# x y z in mm
manual_bregma_targets_by_probe = {
    "pipette": [1, 1, -1],  # in mm!
}


# %% [markdown]
# ## Fit rotation parameters
# Fit the rotation parameters and optionally the scale parameters. If `verbose`
# is set to `True`, the mean and maximum error for each probe will be printed,
# as well as the predicted probe coordinates for each reticle coordinate with
# error for that coordinate.
#
# Note: the reticle coordinates are in mm, as are the probe coordinates. The
# errors are in µm.
#
# The reticle coordinate displayed will NOT have the global offset applied.
# However, the scaling factor will have been applied.
# %%
# Calculate the rotation parameters
reticle_offset = reticle_offsets[reticle_used]
reticle_rotation = reticle_rotations[reticle_used]
if len(manual_calibration_paths) > 0:
    (
        combined_cal_by_probe,
        R_reticle_to_bregma,
        t_reticle_to_bregma,
        combined_pairs_by_probe,
        errs_by_probe,
    ) = debug_parallax_and_manual_calibrations(
        manual_calibration_paths,
        parallax_calibration_paths,
        probes_to_ignore_manual,
    )
else:
    (
        combined_cal_by_probe,
        R_reticle_to_bregma,
        combined_pairs_by_probe,
        errs_by_probe,
    ) = debug_parallax_calibration(
        parallax_calibration_paths[0],
        reticle_offset,
        reticle_rotation,
    )
    t_reticle_to_bregma = reticle_offset
# %% [markdown]
# ## Probe targets in manipulator coordinates
# Get the transformed targets in manipulator coordinates using the fitted
# calibration parameters and the experiment configuration set in the previous
# cells.


# %%
# Print the transformed targets in manipulator coordinates

# Combine the targets and overshoots with the manual targets

dims = ["ML (mm)", "AP (mm)", "DV (mm)"]
combined_targets_and_overshoots_by_probe = {}
for probe, (target_name, overshoot) in targets_and_overshoots_by_probe.items():
    if target_df is None or target_name not in target_df.index:
        logger.warning(f"Target {target_name} not found in target file")
        continue
    target = target_df.loc[target_name, dims].to_numpy().astype(np.float64)
    overshoot_arr = np.array([0, 0, overshoot / 1000])
    combined_targets_and_overshoots_by_probe[probe] = (
        target_name,
        target,
        overshoot_arr,
    )
for probe, target in manual_bregma_targets_by_probe.items():
    target_arr = np.array(target)
    combined_targets_and_overshoots_by_probe[probe] = (
        "Manual",
        target_arr,
        np.array([0, 0, 0]),
    )

zaber_dims = ["X (µm)", "Y (µm)", "Z (µm)"]
output_dims = ["Probe", "Target", "Overshoot (µm)"] + dims + zaber_dims
cols = {}
for probe, (
    target_name,
    target,
    overshoot,
) in combined_targets_and_overshoots_by_probe.items():
    if probe not in combined_cal_by_probe:
        logger.warning(f"Probe {probe} not in calibration files")
        continue
    cols.setdefault("Probe", []).append(probe)
    cols.setdefault("Target", []).append(target_name)
    cols.setdefault("Overshoot (µm)", []).append(1000 * overshoot[2])
    R, t = combined_cal_by_probe[probe]
    probe_target = transform_bregma_to_probe(target, R, t)
    probe_target_and_overshoot = probe_target + overshoot
    target_rnd, probe_target_and_overshoot_rnd = _round_targets(target, probe_target_and_overshoot)
    final_target_bregma = np.round(transform_probe_to_bregma(probe_target_and_overshoot, R, t), 3)
    for dim, dim_val in zip(dims, final_target_bregma):
        cols.setdefault(dim, []).append(np.round(dim_val, 3))
    for dim, dim_val in zip(zaber_dims, probe_target_and_overshoot_rnd):
        cols.setdefault(dim, []).append(dim_val)

df_transformed = pd.DataFrame.from_dict(cols).set_index("Probe").sort_index()
# %%
# Print the targets (AP, ML, DV) and their transformed newscale coordinates (X Y Z)
df_transformed
# %%
if save_path is not None:
    df_transformed.to_csv(save_path)
# %%
